package data_protocol

import (
	"encoding/json"
	"git.sxidc.com/go-tools/utils/strutils"
	"github.com/pkg/errors"
	"time"
)

const (
	DataContentTypeJson = "application/json"
)

func NewCloudEvent(eventID string, eventType string, source string, dataContentType string, data []byte) *CloudEvent {
	return &CloudEvent{
		SpecVersion:     "v1.0",
		ID:              eventID,
		Type:            eventType,
		Source:          source,
		DataContentType: dataContentType,
		Time:            time.Now().Format(time.RFC3339),
		Data:            data,
	}
}

func NewCloudEventJson(eventID string, eventType string, source string, obj any) (*CloudEvent, error) {
	dataJsonBytes, err := json.Marshal(obj)
	if err != nil {
		return nil, errors.New(err.Error())
	}

	return &CloudEvent{
		SpecVersion:     "v1.0",
		ID:              eventID,
		Type:            eventType,
		Source:          source,
		DataContentType: DataContentTypeJson,
		Time:            time.Now().Format(time.RFC3339),
		Data:            dataJsonBytes,
	}, nil
}

func UnmarshalJsonCloudEvent(cloudEventJsonBytes []byte) (*CloudEvent, error) {
	event := new(CloudEvent)
	err := json.Unmarshal(cloudEventJsonBytes, event)
	if err != nil {
		return nil, errors.New(err.Error())
	}

	return event, nil
}

type CloudEvent struct {
	SpecVersion     string `json:"specversion"`
	ID              string `json:"id"`
	Type            string `json:"type"`
	Source          string `json:"source"`
	DataContentType string `json:"datacontenttype"`
	Time            string `json:"time"`
	Data            []byte `json:"data"`
}

func (event *CloudEvent) MarshalJson() ([]byte, error) {
	if strutils.IsStringEmpty(event.SpecVersion) {
		return nil, errors.New("没有传递事件规范版本")
	}

	if strutils.IsStringEmpty(event.ID) {
		return nil, errors.New("没有传递事件ID")
	}

	if strutils.IsStringEmpty(event.Type) {
		return nil, errors.New("没有传递事件类型")
	}

	if strutils.IsStringEmpty(event.Source) {
		return nil, errors.New("没有传递事件源")
	}

	if strutils.IsStringEmpty(event.DataContentType) {
		return nil, errors.New("没有传递事件数据类型")
	}

	if strutils.IsStringEmpty(event.Time) {
		return nil, errors.New("没有传递事件时间")
	}

	eventJson, err := json.Marshal(event)
	if err != nil {
		return nil, errors.New(err.Error())
	}

	return eventJson, nil
}
